
# Coding Tips
##### 15.04.2021
###### Lukas E. G. Fischer
###### Methods Center and Institute of Sports Science • University of Tübingen
###### lukas.fischer@uni-tuebingen.de

### Statistical Software
* **STATA** provides useful out-of-the-box commands for an econometric analysis based on a clean dataset. For
  intensive data shaping and very specific tasks, STATA is not recommended. Also, keep in mind that the
  community is not that big (compared to Python or R). There will not be a solution for every problem you have on
  page 1 of the web search engine of your choice. Moreover, the documentation has room for improvement. Finally,
  there is no university edition for students to download. Hence, you will need to code in the PC-Lab.
* **SAS** is great for data shaping and offers multiple prefabricated options to deal with data. Similar to STATA,
  SAS is not as flexible as a Turing complete language like Python, but fast with respect to execution time and
  coding effort if the underlying problems are not too particular. While SAS is heavily used in Finance, overall,
  the relatively small size of the community is something to consider here as well. However, there are some SAS
  experts at the chair of Econometrics, Statistics and Empirical Economics you can ask for help. As regards content,
  the SAS documentation is very informative. Unfortunately, the documentation is not super beneficial when searching
  for code snippets. There is a university edition available which runs on a virtual machine on any computer, but
  some features (e.g., PROC MODEL) are not available for the university edition.
* **MATLAB** is a matrix-based language with the clearest transition from formula to code (especially for linear
  algebra). MATLAB is not suited to shape the data. As long as the data is clean and numeric, MATLAB is an efficient
  solution. (Starting with Python for data shaping and continuing to work with MATLAB can be a sensible workflow
  depending on the task). In contrast to the latter two programs, MATLAB exhibits a bigger community and provides a
  great documentation. Beside Simulink, the university edition of MATLAB does not have any built-in restrictions.
  Therefore, any student can download the full version and run it on a private computer.
* **R** is a free, highly flexible, and a complete solution for the whole process from data shaping to the
  implementation of statistical models. Since R is matrix-based as well, coding feels similar to MATLAB. There is a
  huge community and a usable documentation for most of the packages.
* **Python** is free and the most popular (powerful) candidate with very well documented packages. Especially for
  Scraping or Machine Learning, you probably want to use Python. Concerning the commands, Python may not be as
  "statistical" as R, but is arguably a "tidier" language.

### Management
* Follow the data science style guide "fischer_ds_style_guide_2021.pdf" in the Literature folder.
* Set up a project in the Code folder using the statistical software of your choice. Then, only use relative paths such
  that the code can run on any computer.
* For large projects, create multiple scripts (data shaping, descriptive statistics, model, ...) and save them in the
  Scripts folder. Edited dataframes and other objects such as lists, dictionaries and so on should be saved in the
  Objects folder. Models can be saved in the folder of the same name.
* Create a text file with useful and often used code snippets (see for example "py_routine_commands.txt").  

### General
* Use sections and paragraphs to structure the code. A good script reads like a well written text. Examples for sections
  are "Load Modules and Functions", "Global Variables", and "Data Loading" (see "eet_main_script.py", for example).
  Paragraphs are lines of code that belong together. Using too many empty lines is as bad as using no empty lines at
  all.
* Add a white space after a comma and avoid long lines (the scripts used here fit an editor window with 120
  columns/characters).
* Create log files for all scripts and store them in the Log_Files folder.
* Use commands or, alternatively, settings (second best solution) to clear console and workspace (variable explorer) at
  the beginning of each run. Do the same for closing all plots.
* Define important variables that are used for the whole script at the beginning (after the START_TIME variable).
  Therefore, they can easily be changed (to provide results for different model specifications, for example). You can
  think about the Global Variables section as the cockpit of your code with the variables defined being the knobs. If
  necessary, use if statements in the later code to change code snippets based on the values of the global variables.
* Use a seed for scripts that contain random components. That way, the results are reproducible. Often, it is sensible
  to set the seed at the beginning of the code. However, there are exceptions depending on the exact structure of the
  code. Let us say you would like to compare two model specifications. The code for the two models is almost the same.
  Hence, you loop over the two specifications to avoid redundancies in the code. For both model specifications, you
  split the data into a train and test set. In this case, the seed needs to be defined within the loop. Otherwise, the
  comparison is based on different samples. Moreover, depending on the statistical software at hand, it can be necessary
  to set module and package specific seeds in addition to the "standard" seed (e.g., when using the Keras API for
  TensorFlow).
* Do not waste time by running every tiny snippet of code. Write a block of code, check for errors and run it. Then,
  move on.
* Try to produce presentable results as fast as possible. Make notes about details to fix and take care of them later.  
* Store your results in variables and create a print section at the end of the code.  
* Comment your code such that you are able to understand it quickly later on (e.g., "drop missing values",
  "print results", ...). Functions should have a description at the beginning to give an understanding what the function
  does. Furthermore, all input and output arguments should be explained.
* Use functions, create own scripts for them (except for functions without input parameters such as eet_calc_returns()),
  and call them in the respective script. It is good practice, to display the input parameters when calling a function (
  write eet_add_numbers(x=2, y=3) instead of eet_add_numbers(2, 3)).
* Avoid loops whenever possible (often the problem can be written efficiently using linear algebra).
* Avoid hardcoding.